﻿using LiveCharts;
using LiveCharts.Configurations;
using LiveCharts.Wpf;
using MathNet.Numerics;
using Misc;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Data;
using System.Windows.Media;

namespace FLY.Thick.FilmCasting.UI.UiModule
{
    public class TempScanGraphVmUt : INotifyPropertyChanged
    {
        #region 图表控制
        public double YMax { get; set; }
        public double YMin { get; set; }
        public double Target { get; set; }
        public double Tolerance2YMax { get; set; }
        public double Tolerance2YMin { get; set; }
        public double ToleranceYMax { get; set; }
        public double ToleranceYMin { get; set; }
        public double XMax { get; set; }
        public double XMin { get; set; }

        public int XStep { get; set; }

        public Func<double, string> YFormatter { get; set; }
        public Func<double, string> XFormatter { get; set; }
        public SeriesCollection Series { get; set; }

        #endregion

        #region 颜色
        List<Brush> AreaColors;
        #endregion


        #region 界面统计值
        /// <summary>
        /// 平均值
        /// </summary>
        public double Average { get; private set; } = double.NaN;

        public double Sigma2 { get; private set; } = double.NaN;
        /// <summary>
        /// 最大值
        /// </summary>
        public double Max { get; private set; } = double.NaN;
        /// <summary>
        /// 最小值
        /// </summary>
        public double Min { get; private set; } = double.NaN;

        public string Sigma2Text { get; private set; }
        public string MaxText { get; private set; }
        public string MinText { get; private set; }

        /// <summary>
        /// 混合数
        /// </summary>
        public int Mix { get; private set; } = 2;

        /// <summary>
        /// 测量时间
        /// </summary>
        public DateTime Time { get; private set; }

        /// <summary>
        /// 数据库中的序号
        /// </summary>
        public long Id { get; private set; }

        public double Velocity { get; set; }
        public double Position { get; set; }

        const int NULL_VALUE = -100000;

        /// <summary>
        /// 中心数据序号
        /// </summary>
        public int MidIndex => BoltCnt / 2;
        /// <summary>
        /// 总数量
        /// </summary>
        public int BoltCnt { get; set; } = 100;

        public int DataBegin { get; set; } = NULL_VALUE;
        public int DataEnd { get; set; } = NULL_VALUE;


        #endregion



        ChartValues<double> Values { get; } = new ChartValues<double>();
        object Mapper { get; set; }

        bool IsPercent = true;
        bool IsAutoTarget = true;

        public TempScanGraphVmUt()
        {

            #region 与数据无关界面参数
            AreaColors = FLY.Thick.FilmCasting.UI.Themes.Colors.AreaColors;
            YFormatter = (y) =>
            {
                if (IsPercent)
                {
                    if (y == Target)
                    {
                        return y.ToString("F1");
                    }
                    else
                    {
                        double percent = 100.0 * (y - Target) / Target;
                        if (percent > 0)
                        {
                            return $"+{Math.Abs(percent):F1}%";
                        }
                        else
                        {
                            return $"-{Math.Abs(percent):F1}%";
                        }

                    }
                }
                else
                {
                    return y.ToString("F1");
                }
            };

            XFormatter = (x) =>
            {
                if (x == 0)
                    return "0mm";
                else
                    return $"{x * 10:F0}";
            };
            Mapper = Mappers.Xy<double>()
                .X((value, index) =>
                {
                    return (index - MidIndex);//单位cm
                })
                .Y(value => value)
                .Fill((value) =>
                {
                    if ((value > Tolerance2YMax) || (value < Tolerance2YMin))
                    {
                        return AreaColors[2];
                    }
                    else if ((value > ToleranceYMax) || (value < ToleranceYMin))
                    {
                        return AreaColors[1];
                    }
                    else
                    {
                        return AreaColors[0];
                    }
                });
                  Series = new SeriesCollection
            {
                new LineSeries
                {
                    Values = Values,
                    StrokeThickness = 3,
                    Stroke = AreaColors[0],
                    Fill =AreaColors[4],
                    PointGeometry = null,
                    Configuration = Mapper
                }
            };
            #endregion
            #region 数据
            int scanWidth = 2300;
            int dataWidth = 2000;
            int filmBegin = -1100;
            int filmEnd = 1100;
            double target = 150;
            double tolerance = target * 0.02;
            double yrangepercent = 3;
            int nbolts = 251;
            double[] datas = new double[nbolts];
            Random random = new Random();
            for (int i = 0; i < nbolts; i++)
            {
                datas[i] = (Math.Sin(i * Math.PI / nbolts) * 3) * tolerance + target + (random.NextDouble() - 0.5) * 1 - 6;
            }
            int midMm = nbolts * 10 / 2;
            int filmBeginIndex = (filmBegin + midMm) / 10;
            int filmEndIndex = (filmEnd + midMm) / 10;
            for (int i = 0; i < filmBeginIndex; i++) {
                datas[i] = double.NaN;
            }
            for (int i = filmEndIndex; i < nbolts; i++)
            {
                datas[i] = double.NaN;
            }

            #endregion
            #region 界面数据


            BoltCnt = nbolts;
            XMin = -scanWidth /10/ 2;
            XMax = scanWidth /10/ 2;
            XStep = 10;
            Mix = 2;
            Time = DateTime.Now;
            Id = 312312;
            DataBegin = filmBegin;
            DataEnd = filmEnd;
            Velocity = 40.1;
            Position = 800.4;

            IsPercent = true;
            IsAutoTarget = true;

            Values.Clear();
            Values.AddRange(datas);


            double[] values = datas;
            int midIndex = MidIndex;
            int beginIndex = midIndex - dataWidth / 2 / 10;
            int endIndex = midIndex + dataWidth / 2 / 10;
            List<double> list = new List<double>();
            for (int i = beginIndex; i <= endIndex; i++)
            {
                if (i < 0)
                    i = 0;
                if (i >= values.Count())
                    break;
                double v = values[i];
                if (!double.IsNaN(v))
                    list.Add(values[i]);
            }

            if (list.Count() > 0)
            {
                Average = list.Average();
                Max = list.Max();
                Min = list.Min();
                Sigma2 = list.Sigma() * 2;
            }
            else
            {
                Average = double.NaN;
                Max = double.NaN;
                Min = double.NaN;
                Sigma2 = double.NaN;
            }


            if (IsPercent)
            {
                MaxText = $"+{((Max - Average) / Average) * 100.0:F1}%";
                MinText = $"-{((Average - Min) / Average) * 100.0:F1}%";
                Sigma2Text = $"{(Sigma2 / Average) * 100.0:F1}%";
            }
            else
            {
                MaxText = $"{Max:F1}";
                MinText = $"{Min:F1}";
                Sigma2Text = $"{Sigma2:F1}";
            }

            if (IsPercent || IsAutoTarget)
            {
                Target = Average;
            }
            else
            {
                Target = target;
            }

            ToleranceYMax = Target + tolerance;
            ToleranceYMin = Target - tolerance;
            Tolerance2YMax = Target + tolerance * 2;
            Tolerance2YMin = Target - tolerance * 2;
            YMax = Target + tolerance * yrangepercent;
            YMin = Target - tolerance * yrangepercent;

            #endregion


            ((Series)Series[0]).SetBinding(Column2Series.YAxisCrossingProperty, new Binding(nameof(Target)) { Source = this });
        }

        public event PropertyChangedEventHandler PropertyChanged;
    }
}
